/*
 * Copyright (C) 1997, 2011 LAAS/CNRS -- RIA --
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * Maher Khatib - Mon Jun  9 1997
 *
 */
#ifndef _SERIAL_H_
#define _SERIAL_H_

/* Consts */
#define READ_TASK_STACK_LENGTH                 (0x4000)
#define WRITE_TASK_STACK_LENGTH  READ_TASK_STACK_LENGTH

/*
 * serial buffer state
 */
typedef enum serial_buffer_state {
  SERIAL_BUFFER_OK,
  SERIAL_BUFFER_ERROR,
  SERIAL_BUFFER_FULL
} SERIAL_BUFFER_STATE;

/*
 * internal serial buffer control
 */

typedef struct serial_buffer_control_str {
  char *buf;
  char *sp;
  size_t size;
  size_t available;
  int task_id;
  SERIAL_BUFFER_STATE buf_state;
} SERIAL_BUFFER_CONTROL_STR;


/*
 * Serial str
 */

typedef struct serial_buffer_str {
  int fd;
  int period;
  SERIAL_BUFFER_CONTROL_STR in;
  SERIAL_BUFFER_CONTROL_STR out;
} SERIAL_BUFFER_STR;

/* baud rates */
#define SERIAL_BAUD_RATE_MIN   ( 9600)
#define SERIAL_BAUD_RATE_MAX   (58800)

/* errors */
#define M_serial                 (300 << 16)
#define S_serial_cannot_access_semaphore            (M_serial | 1)
#define S_serial_bad_baud_rate                      (M_serial | 2)
#define S_serial_serial_not_initialized             (M_serial | 3)
#define S_serial_bad_device_name                    (M_serial | 4)
#define S_serial_scroll_too_big                     (M_serial | 5)

#ifdef __cplusplus
extern "C" {
#endif
    
extern STATUS serial_change_baud_rate ( SERIAL_BUFFER_STR *s, int baud );
extern STATUS serial_flush_in_buffer ( SERIAL_BUFFER_STR *s );
extern STATUS serial_free ( SERIAL_BUFFER_STR *s );
extern SERIAL_BUFFER_STATE serial_get_in_buffer_state ( SERIAL_BUFFER_STR *s );
extern SERIAL_BUFFER_STATE serial_get_out_buffer_state ( SERIAL_BUFFER_STR *s );
extern SERIAL_BUFFER_STR * serial_init ( char *device, int baud, size_t size_in, size_t size_out, int priority_in, int priority_out );
extern int serial_in_buf_show ( SERIAL_BUFFER_STR *s );
extern int serial_in_get_n_chars ( SERIAL_BUFFER_STR *s );
extern int serial_out_buf_show ( SERIAL_BUFFER_STR *s );
extern int serial_read_buffer ( SERIAL_BUFFER_STR *s, char *buf, int n );
extern int serial_read_buffer_without_scroll ( SERIAL_BUFFER_STR *s, char *buf, int n );
extern int serial_scroll_in_buffer ( SERIAL_BUFFER_STR *s, int n );
extern STATUS serial_set_option ( SERIAL_BUFFER_STR *s, int option );
extern int serial_test ( char *device, int rate );
extern void serial_test_stop ( void );
extern void serial_test_write ( char *data );
extern int serial_write_to_buffer ( SERIAL_BUFFER_STR *s, char *data, int size );

#ifdef __cplusplus
};
#endif

#endif /* _SERIAL_H_ */
