/*
 * Copyright (c) 1997, 2011 CNRS-LAAS 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* ----------------------------------------------------------------------
 * tserie (<debit>)
 * 
 * Fonction basic qui lit ce qu'il y a sur la ligne serie
 * et affiche ce qu'elle peut sur 2 colonnes :
 * 8 bits en binaire a gauche / traduction en ascii a droite (ou '.')
 * 
 * Mise en oeuvre :
 *     Mettre a jour le nom du port (macro 'PORT' ci-dessous)
 *     ccvx test-serie.c
 *     ld < /hilare2/src/tools/serial/test-serie.o
 *     tserie 9600
 */


#include <vxWorks.h>
#include <stdio.h>
#include <ioLib.h>
#include <ctype.h>

/* Nom du port ou lire les donnees */
#define PORT "/tyCo/1"

#define SERIAL_BAUD_RATE_MIN   ( 9600)
#define SERIAL_BAUD_RATE_MAX   (58800)

/* Nombre de caracteres affiches par colonnes */
#define NB_ELT 8

int stop = 0;
#define MSG_LEN 8

static int serial_init (char *device, int rate);
static int serialFd=NULL;


/* ----------------------------------------------------------------------
 * tserie - la fonction principale de lecture des donnees 
 *          sur le port serie
 */
void tserie (int baud)
{
  char buf[MSG_LEN];
  int count=0;
  int status;
  int cp=0, i;
  char mem[NB_ELT];

  if (serialFd) 
    close(serialFd);

  /* Init */
  if (!(serialFd = serial_init(PORT, baud))) {
    if (serialFd) {
      close(serialFd);
    }
    return;
  }
  memset(buf, 0, MSG_LEN);
  cp = 0;

  /* Boucle de lecture */
  while(!stop) {
    
    /* lecture */
    memset(buf, 0, MSG_LEN);
    status = read(serialFd, &buf[count], 1);

    if (status != 1) {
      perror("read");
      continue;
    }

    /* memorisation de NB_ELT avant affichage */
    mem[cp] = buf[0];
    cp++;

    /* Affichage d'une ligne de NB_ELT valeurs */
    if (cp>=NB_ELT) {

      /* hexa */
      for (i=0; i<NB_ELT; i++)
	printf ("%02x ", (unsigned char)mem[i]);
      printf ("   ");

      /* charactere */
      for (i=0; i<NB_ELT; i++) {
	if (mem[i] > 32 && mem[i]<127)
	  printf ("%c", (unsigned char)mem[i]);
	else
	  printf (".");
      }

      printf ("\n");
      cp = 0;
    }

  } /* while */

  /* On affiche ce qui reste avant de sortir */
  /* hexa */
  for (i=0; i<cp; i++)
    printf ("%02x ", (unsigned char)mem[i]);
  printf ("   ");

  /* charactere */
  for (i=0; i<cp; i++) {
    if (mem[i] > 32 && mem[i]<127)
      printf ("%c", (unsigned char)mem[i]);
    else
      printf (".");
  }
  printf ("\n");

  /* Fin */
  close(serialFd);
}



/*
 * Initialize sick serial communications
 */
static int
serial_init (char *device, int baud)
{

  /* check device */
  if (!device) {
    printf ("device ?\n");
    return NULL;
  }

  /* check rate */
  if (baud < 0 || baud > SERIAL_BAUD_RATE_MAX) {
    printf ("baud %d ?\n", baud);
    return NULL;
  }

  /* open device */
  serialFd = open(device, O_RDWR, 0);

  /* check open */
  if (serialFd == -1) {
    printf ("cannot open device");
    return NULL;
  }

  /* speed config.  */
  if (ioctl(serialFd, FIOBAUDRATE, baud) < 0) {
    perror("set baud rate");
    close(serialFd);
    return NULL;
  }
  
  /* set raw option */
  if (ioctl(serialFd, FIOSETOPTIONS, OPT_RAW) < 0) {
    perror("set options OPT_RAW");
    close(serialFd);
    return NULL;
  }

  /* Flush physical buffer */
  if (ioctl (serialFd, FIOFLUSH, 0) < 0) {
    perror("Cannot flush IO buffer...");
    close(serialFd);
    return NULL;
  }

  /* Init OK */
  return serialFd;
}
